@@ -19,6 +19,16 @@ from .extensions import (
     gettext,
     lazy_gettext as _
 )
+from . import types
+
+try:
+    import arrow
+except ImportError:
+    arrow = None
+try:
+    from sqlalchemy_utils.types import ArrowType
+except ImportError:
+    ArrowType = None
 
 
 class UnrecognizedOperator(ValueError):
@@ -299,6 +309,27 @@ class FilterBase(object):
         """
         return None
 
+    def serialize_filter_operator(self, op):
+        return types.FilterOperator(
+            key=op.key,
+            label=op.display,
+            field_type=op.field_type,
+            hint=op.hint,
+        )
+
+    def serialize_filter_option(self, key, value):
+        return types.FilterOption(
+            key=key,
+            value=value,
+        )
+
+    def serialize_filter_spec(self):
+        return types.FilterSpec(
+            operators=[self.serialize_filter_operator(op) for op in self.operators],
+            primary_op=self.serialize_filter_operator(
+                self.primary_op) if self.primary_op else None,
+        )
+
     def new_instance(self, **kwargs):
         """
         Note: Ensure any overrides of this method accept and pass through kwargs to preserve
@@ -358,6 +389,16 @@ class OptionsFilterBase(FilterBase):
         self._options_seq = None
         self._options_keys = None
 
+    def serialize_filter_spec(self):
+        base_spec = super().serialize_filter_spec()
+        return types.OptionsFilterSpec(
+            operators=base_spec.operators,
+            primary_op=base_spec.primary_op,
+            options=[
+                self.serialize_filter_option(key, value)
+                for key, value in self.options_seq],
+        )
+
     def new_instance(self, **kwargs):
         filter = FilterBase.new_instance(self, **kwargs)
         filter.setup_validator()
@@ -534,6 +575,8 @@ class OptionsEnumFilter(OptionsFilterBase):
     Notable args:
         enum_type (Enum): Python Enum type to use for options list.
     """
+    enum_type = None
+
     def __init__(
             self,
             sa_col,
@@ -543,7 +586,7 @@ class OptionsEnumFilter(OptionsFilterBase):
             default_value2=None,
             enum_type=None,
     ):
-        self.enum_type = enum_type
+        self.enum_type = enum_type or self.__class__.enum_type
 
         if self.enum_type is None:
             raise ValueError('enum_type argument not given')
@@ -775,13 +818,13 @@ class _DateMixin(object):
             ops.not_between.key
         ):
             # !!!: localize
-            self.value1_set_with = self.value1.strftime('%m/%d/%Y')
+            self.value1_set_with = self.value1.strftime('%Y-%m-%d')
         if isinstance(self.value2, dt.date) and self.op in (
             ops.between.key,
             ops.not_between.key
         ):
             # !!!: localize
-            self.value2_set_with = self.value2.strftime('%m/%d/%Y')
+            self.value2_set_with = self.value2.strftime('%Y-%m-%d')
 
     def _between_range(self):
         if self.value1 is None:
@@ -1023,6 +1066,14 @@ class DateFilter(_DateOpQueryMixin, _DateMixin, FilterBase):
         ops.last_month, ops.this_year
     )
     input_types = 'input', 'select', 'input2'
+    html_input_types = {
+        ops.eq: 'date',
+        ops.not_eq: 'date',
+        ops.less_than_equal: 'date',
+        ops.greater_than_equal: 'date',
+        ops.between: 'date',
+        ops.not_between: 'date',
+    }
 
     def __init__(self, sa_col, _now=None, default_op=None, default_value1=None,
                  default_value2=None):
@@ -1037,6 +1088,14 @@ class DateFilter(_DateOpQueryMixin, _DateMixin, FilterBase):
         self._was_time_given1 = False
         self._was_time_given2 = False
 
+        self.check_arrow_type()
+
+    def check_arrow_type(self):
+        """Verify that the expression given to the filter is not ArrowType. If it
+        is, cast it to a date to avoid type problems in the date filter"""
+        if arrow and ArrowType and isinstance(self.sa_col.type, ArrowType):
+            self.sa_col = sa.sql.cast(self.sa_col, sa.Date)
+
     def set(self, op, value1, value2=None):
         super(DateFilter, self).set(op, value1, value2)
         self.format_display_vals()
@@ -1147,6 +1206,14 @@ class DateTimeFilter(DateFilter):
         _now (datetime, optional): Useful for testing, supplies a date the filter will use instead
         of the true `datetime.now()`. Defaults to None.
     """
+    html_input_types = {
+        ops.eq: 'datetime-local',
+        ops.not_eq: 'datetime-local',
+        ops.less_than_equal: 'datetime-local',
+        ops.greater_than_equal: 'datetime-local',
+        ops.between: 'datetime-local',
+        ops.not_between: 'datetime-local',
+    }
     op_to_query = ImmutableDict({**DateFilter.op_to_query, **{
         ops.today: lambda self, query, today: query.filter(self.sa_col.between(
             ensure_datetime(today),
@@ -1209,10 +1276,10 @@ class DateTimeFilter(DateFilter):
             )
         ),
         ops.eq: lambda self, query, today: (
-            query.filter(self._eq_clause()) if self._has_date_only1 else None
+            query.filter(self._eq_clause())
         ),
         ops.not_eq: lambda self, query, today: (
-            query.filter(~self._eq_clause()) if self._has_date_only1 else None
+            query.filter(~self._eq_clause())
         ),
         ops.less_than_equal: lambda self, query, today: query.filter(
             self.sa_col <= ensure_datetime(
@@ -1230,6 +1297,10 @@ class DateTimeFilter(DateFilter):
                                              default_value1=default_value1,
                                              default_value2=default_value2)
 
+    def check_arrow_type(self):
+        """DateTimeFilter has no problems with ArrowType. Pass this case through."""
+        pass
+
     def format_display_vals(self):
         ops_single_val = (
             ops.eq.key,
@@ -1243,16 +1314,16 @@ class DateTimeFilter(DateFilter):
         )
         if isinstance(self.value1, dt.datetime) and self.op in ops_single_val + ops_double_val:
             # !!!: localize
-            self.value1_set_with = self.value1.strftime('%m/%d/%Y %I:%M %p')
+            self.value1_set_with = self.value1.strftime('%Y-%m-%dT%H:%M')
             if self.op in ops_single_val and self._has_date_only1:
                 # !!!: localize
-                self.value1_set_with = self.value1.strftime('%m/%d/%Y')
+                self.value1_set_with = self.value1.strftime('%Y-%m-%d')
         if isinstance(self.value2, dt.datetime) and self.op in ops_double_val:
             # !!!: localize
-            self.value2_set_with = self.value2.strftime('%m/%d/%Y %I:%M %p')
+            self.value2_set_with = self.value2.strftime('%Y-%m-%dT%H:%M')
             if self._has_date_only2:
                 # !!!: localize
-                self.value2_set_with = self.value2.strftime('%m/%d/%Y 11:59 PM')
+                self.value2_set_with = self.value2.strftime('%Y-%m-%dT23:59')
 
     def _between_clause(self):
         if self._has_date_only2:
@@ -1263,9 +1334,14 @@ class DateTimeFilter(DateFilter):
         return self.sa_col.between(ensure_datetime(self.value1), right_side)
 
     def _eq_clause(self):
-        left_side = ensure_datetime(self.value1.date())
-        right_side = ensure_datetime(self.value1.date(), time_part=dt.time(23, 59, 59, 999999))
-        return self.sa_col.between(left_side, right_side)
+        if self._has_date_only1:
+            left_side = ensure_datetime(self.value1.date())
+            right_side = ensure_datetime(self.value1.date(), time_part=dt.time(23, 59, 59, 999999))
+            return self.sa_col.between(left_side, right_side)
+        if self.value1 and self.value1.microsecond == 0:
+            left_side = self.value1
+            right_side = self.value1 + dt.timedelta(microseconds=999999)
+            return self.sa_col.between(left_side, right_side)
 
     def _process_datetime(self, value, is_value2):
         try:
@@ -1339,9 +1415,17 @@ class TimeFilter(FilterBase):
     operators = (ops.eq, ops.not_eq, ops.less_than_equal, ops.greater_than_equal, ops.between,
                  ops.not_between, ops.empty, ops.not_empty)
     input_types = 'input', 'input2'
+    html_input_types = {
+        ops.eq: 'time',
+        ops.not_eq: 'time',
+        ops.less_than_equal: 'time',
+        ops.greater_than_equal: 'time',
+        ops.between: 'time',
+        ops.not_between: 'time',
+    }
 
     # !!!: localize
-    time_format = '%I:%M %p'
+    time_format = '%H:%M'
 
     def apply(self, query):
         if self.op == self.default_op and self.value1 is None:
@@ -1357,14 +1441,22 @@ class TimeFilter(FilterBase):
 
         # Casting this because some SQLAlchemy dialects (MSSQL) convert the value to datetime
         # before binding.
-        val = sa.cast(self.value1, sa.Time)
+        endval = val = sa.cast(self.value1, sa.Time)
+        if self.value1:
+            endval = sa.cast(
+                (
+                    dt.datetime.combine(dt.date.today(), self.value1)
+                    + dt.timedelta(seconds=59, microseconds=999999)
+                ).time(),
+                sa.Time
+            )
 
         if self.op == ops.eq:
-            query = query.filter(self.sa_col == val)
+            query = query.filter(self.sa_col.between(val, endval))
         elif self.op == ops.not_eq:
-            query = query.filter(self.sa_col != val)
+            query = query.filter(~self.sa_col.between(val, endval))
         elif self.op == ops.less_than_equal:
-            query = query.filter(self.sa_col <= val)
+            query = query.filter(self.sa_col <= endval)
         elif self.op == ops.greater_than_equal:
             query = query.filter(self.sa_col >= val)
         else:
