@@ -17,18 +17,21 @@ from pyquery import PyQuery
 from webgrid import (
     BoolColumn,
     Column,
+    ColumnGroup,
     DateTimeColumn,
     LinkColumnBase,
     NumericColumn,
     YesNoColumn,
     col_filter,
     col_styler,
+    extensions,
     row_styler,
 )
-from webgrid.filters import TextFilter, OptionsEnumFilter
+from webgrid.filters import TextFilter, OptionsEnumFilter, DateFilter
 from webgrid.renderers import (
     CSV,
     HTML,
+    JSON,
     XLS,
     XLSX,
     RenderLimitExceeded,
@@ -50,7 +53,7 @@ from webgrid_ta.model.entities import (
     db,
 )
 
-from .helpers import eq_html, inrequest, render_in_grid
+from .helpers import eq_html, _inrequest, render_in_grid
 
 
 def _query_exclude_person(query):
@@ -212,14 +215,14 @@ def assert_tag(html, tag, text=None, **kwargs):
 
 class TestHtmlRenderer(object):
     key_data = (
-        {'id': 1, 'name': 'one'},
-        {'id': 2, 'name': 'two'},
-        {'id': 3, 'name': 'three'},
-        {'id': 4, 'name': 'three'},
-        {'id': 5, 'name': 'three'},
+        {'id': 1, 'name': 'one', 'status': 'new', 'emails': ''},
+        {'id': 2, 'name': 'two', 'status': 'new', 'emails': ''},
+        {'id': 3, 'name': 'three', 'status': 'pending', 'emails': ''},
+        {'id': 4, 'name': 'three', 'status': 'pending', 'emails': ''},
+        {'id': 5, 'name': 'three', 'status': 'complete', 'emails': ''},
     )
 
-    @inrequest('/')
+    @_inrequest('/')
     def test_car_html(self):
         key_data = (
             {'id': 1, 'make': 'ford', 'model': 'F150&', 'color': 'pink',
@@ -235,19 +238,19 @@ class TestHtmlRenderer(object):
         eq_html(mg.html.table(), 'basic_table.html')
 
     @pytest.mark.skipif(db.engine.dialect.name != 'sqlite', reason="IDs will not line up")
-    @inrequest('/')
+    @_inrequest('/')
     def test_people_html(self):
         pg = render_in_grid(PeopleGrid, 'html')()
         eq_html(pg.html.table(), 'people_table.html')
 
     @pytest.mark.skipif(db.engine.dialect.name != 'sqlite', reason="IDs will not line up")
-    @inrequest('/')
+    @_inrequest('/')
     def test_stopwatch_html(self):
         # Test Stopwatch grid with column groups.
         grid = StopwatchGrid()
         eq_html(grid.html.table(), 'stopwatch_table.html')
 
-    @inrequest('/')
+    @_inrequest('/')
     def test_default_jinja_env(self):
         class TGrid(Grid):
             manager = None
@@ -280,7 +283,7 @@ class TestHtmlRenderer(object):
         assert isinstance(result, Markup)
         assert result == ' bool1 empty="" esc&amp;="&lt;&gt;&#34;" text="abc"'
 
-    @inrequest('/')
+    @_inrequest('/')
     def test_no_filters(self):
         class TGrid(Grid):
             Column('Test', Person.id)
@@ -288,7 +291,7 @@ class TestHtmlRenderer(object):
         tg = TGrid()
         assert 'Add Filter' not in tg.html()
 
-    @inrequest('/')
+    @_inrequest('/')
     def test_hide_excel_deprecated(self):
         class TGrid(Grid):
             hide_excel_link = True
@@ -305,34 +308,34 @@ class TestHtmlRenderer(object):
         g.apply_qs_args()
         return g
 
-    @inrequest('/thepage?perpage=5&onpage=1')
+    @_inrequest('/thepage?perpage=5&onpage=1')
     def test_current_url(self):
         g = self.get_grid()
         assert '/thepage?onpage=1&perpage=5' == g.html.current_url()
         assert '/thepage?onpage=1&perpage=10' == g.html.current_url(perpage=10)
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_current_url_qs_prefix(self):
         g = self.get_grid(qs_prefix='dg_')
         assert '/thepage?dg_perpage=10' == g.html.current_url(perpage=10)
 
-    @inrequest('/thepage?perpage=5&onpage=1&dgreset=1')
+    @_inrequest('/thepage?perpage=5&onpage=1&dgreset=1')
     def test_current_url_reset_removed(self):
         g = self.get_grid()
         assert '/thepage?onpage=1&perpage=5' == g.html.current_url()
         assert '/thepage?onpage=1&perpage=10' == g.html.current_url(perpage=10)
 
-    @inrequest('/thepage?foo_dgreset=1')
+    @_inrequest('/thepage?foo_dgreset=1')
     def test_current_url_reset_removed_prefix(self):
         g = self.get_grid(qs_prefix='foo_')
         assert '/thepage?foo_perpage=5' == g.html.current_url(perpage=5)
 
-    @inrequest('/thepage?perpage=5&onpage=1')
+    @_inrequest('/thepage?perpage=5&onpage=1')
     def test_current_url_reset_added(self):
         g = self.get_grid()
         assert '/thepage?dgreset=1&onpage=1&perpage=5' == g.html.current_url(dgreset=1)
 
-    @inrequest('/thepage?perpage=5&onpage=1')
+    @_inrequest('/thepage?perpage=5&onpage=1')
     def test_xls_url(self):
         g = self.get_grid()
         with pytest.warns(DeprecationWarning,
@@ -340,35 +343,35 @@ class TestHtmlRenderer(object):
             url = g.html.xls_url()
         assert url == '/thepage?export_to=xls&onpage=1&perpage=5'
 
-    @inrequest('/thepage?perpage=5&onpage=1')
+    @_inrequest('/thepage?perpage=5&onpage=1')
     def test_export_url(self):
         g = self.get_grid()
         assert g.html.export_url('xlsx') == '/thepage?export_to=xlsx&onpage=1&perpage=5'
         assert g.html.export_url('xls') == '/thepage?export_to=xls&onpage=1&perpage=5'
         assert g.html.export_url('csv') == '/thepage?export_to=csv&onpage=1&perpage=5'
 
-    @inrequest('/thepage?onpage=3')
+    @_inrequest('/thepage?onpage=3')
     def test_paging_url_first(self):
         g = self.get_grid()
         assert g.html.paging_url_first() == '/thepage?onpage=1&perpage=1'
 
-    @inrequest('/thepage?onpage=3')
+    @_inrequest('/thepage?onpage=3')
     def test_paging_url_next(self):
         g = self.get_grid()
         assert g.html.paging_url_next() == '/thepage?onpage=4&perpage=1'
 
-    @inrequest('/thepage?onpage=3')
+    @_inrequest('/thepage?onpage=3')
     def test_paging_url_prev(self):
         g = self.get_grid()
         assert g.html.paging_url_prev() == '/thepage?onpage=2&perpage=1'
 
-    @inrequest('/thepage?onpage=3')
+    @_inrequest('/thepage?onpage=3')
     def test_paging_url_last(self):
         g = self.get_grid()
         assert g.html.paging_url_last() == '/thepage?onpage=5&perpage=1'
 
-    @inrequest('/thepage?foo=bar&onpage=5&perpage=10&sort1=1&sort2=2&sort3=3&op(name)=eq&v1(name)'
-               '=bob&v2(name)=fred')
+    @_inrequest('/thepage?foo=bar&onpage=5&perpage=10&sort1=1&sort2=2&sort3=3&op(name)=eq&v1(name)'
+                '=bob&v2(name)=fred&search=bar')
     def test_reset_url(self):
         g = self.get_grid()
         assert (
@@ -376,7 +379,7 @@ class TestHtmlRenderer(object):
             == '/thepage?dgreset=1&foo=bar&session_key={0}'.format(g.session_key)
         )
 
-    @inrequest('/thepage?foo=bar&onpage=5')
+    @_inrequest('/thepage?foo=bar&onpage=5')
     def test_form_action_url(self):
         g = self.get_grid()
         assert (
@@ -384,6 +387,36 @@ class TestHtmlRenderer(object):
             == '/thepage?foo=bar&session_key={0}'.format(g.session_key)
         )
 
+    @_inrequest('/thepage?foo=bar&onpage=5')
+    def test_form_action_method_get(self):
+        g = self.get_grid()
+        assert g.html.form_action_method() == 'get'
+
+        pyq = PyQuery(g.html())
+        assert pyq('form.header').attr('method') == 'get'
+        assert not pyq('form.header > input[name="csrf_token"]')
+
+    @_inrequest('/thepage?foo=bar&onpage=5')
+    def test_form_action_method_post(self):
+        class TestManager(SimpleGrid.manager.__class__):
+            args_loaders = (
+                extensions.RequestArgsLoader,
+                extensions.RequestFormLoader,
+            )
+
+        class TGrid(Grid):
+            manager = TestManager()
+
+            Column('ID', 'id')
+
+        g = TGrid()
+        g.set_records(self.key_data)
+        assert g.html.form_action_method() == 'post'
+
+        pyq = PyQuery(g.html())
+        assert pyq('form.header').attr('method') == 'post'
+        assert pyq('form.header > input[name="csrf_token"]')
+
     @pytest.mark.parametrize('page_param,input_value', [
         (1, '1'),
         (2, '2'),
@@ -396,7 +429,7 @@ class TestHtmlRenderer(object):
     ])
     def test_paging_select(self, page_param, input_value):
 
-        @inrequest('/thepage?onpage={}'.format(page_param))
+        @_inrequest('/thepage?onpage={}'.format(page_param))
         def check_paging():
             g = self.get_grid()
             select_html = g.html.paging_select()
@@ -406,7 +439,7 @@ class TestHtmlRenderer(object):
 
         check_paging()
 
-    @inrequest('/thepage?onpage=2')
+    @_inrequest('/thepage?onpage=2')
     def test_paging_html(self):
         g = self.get_grid()
         input_html = g.html.paging_input()
@@ -465,7 +498,7 @@ class TestHtmlRenderer(object):
         assert g.html.paging_img_next_dead() in footer_html
         assert g.html.paging_img_last_dead() in footer_html
 
-    @inrequest('/thepage?sort1=name&sort2=-id')
+    @_inrequest('/thepage?sort1=name&sort2=-id')
     def test_sorting_html(self):
         g = self.get_grid()
 
@@ -488,20 +521,20 @@ class TestHtmlRenderer(object):
         assert 'sort-asc' not in heading_row
         assert 'sort-desc' not in heading_row
 
-    @inrequest('/thepage?sort1=name')
+    @_inrequest('/thepage?sort1=name')
     def test_sorting_headers_asc(self):
         g = self.get_grid()
         heading_row = g.html.table_column_headings()
         assert_tag(heading_row, 'a', text='Name', class_='sort-asc', href='/thepage?sort1=-name')
 
-    @inrequest('/thepage?sort1=-name')
+    @_inrequest('/thepage?sort1=-name')
     def test_sorting_headers_desc(self):
         g = self.get_grid()
         heading_row = g.html.table_column_headings()
         assert_tag(heading_row, 'a', text='Name', class_='sort-desc', href='/thepage?sort1=name')
 
-    @inrequest('/thepage?op(firstname)=eq&v1(firstname)=foo&op(createdts)=between&v1(createdts)='
-               '2%2F15%2F12&&v2(createdts)=2012-02-16')
+    @_inrequest('/thepage?op(firstname)=eq&v1(firstname)=foo&op(createdts)=between&v1(createdts)='
+                '2%2F15%2F12&&v2(createdts)=2012-02-16')
     def test_filtering_input_html(self):
         g = PeopleGrid()
 
@@ -525,13 +558,13 @@ class TestHtmlRenderer(object):
 
         filter_html = g.html.filtering_col_inputs1(g.key_column_map['createdts'])
         assert '<input id="createdts_input1" name="v1(createdts)" type="text" value=' + \
-            '"02/15/2012 12:00 AM" />' in filter_html, filter_html
+            '"2012-02-15T00:00" />' in filter_html, filter_html
 
         filter_html = g.html.filtering_col_inputs2(g.key_column_map['createdts'])
         assert '<input id="createdts_input2" name="v2(createdts)" type="text" value=' + \
-            '"02/16/2012 11:59 PM" />' in filter_html, filter_html
+            '"2012-02-16T23:59" />' in filter_html, filter_html
 
-    @inrequest('/thepage?op(firstname)=foobar&v1(firstname)=baz')
+    @_inrequest('/thepage?op(firstname)=foobar&v1(firstname)=baz')
     def test_filtering_invalid_operator(self):
         g = PeopleGrid()
 
@@ -539,14 +572,14 @@ class TestHtmlRenderer(object):
         assert '<input id="firstname_input1" name="v1(firstname)" type="text" />' in filter_html, \
             filter_html
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_extra_filter_attrs(self):
         g = PeopleGrid()
         g.key_column_map['firstname'].filter.html_extra = {'data-special-attr': 'foo'}
         filter_html = g.html.filtering_table_row(g.key_column_map['firstname'])
         assert '<tr class="firstname_filter" data-special-attr="foo">' in filter_html, filter_html
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_filter_primary_op_specified(self):
         g = PeopleGrid()
         g.key_column_map['firstname'].filter.primary_op = '!eq'
@@ -554,7 +587,7 @@ class TestHtmlRenderer(object):
         assert PyQuery(filter_html).find('option[value="!eq"]').attr('data-render') == 'primary'
         assert not PyQuery(filter_html).find('option[value="eq"]').attr('data-render')
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_filter_primary_op_not_specified(self):
         g = PeopleGrid()
         g.key_column_map['firstname'].filter.primary_op = None
@@ -562,7 +595,7 @@ class TestHtmlRenderer(object):
         assert PyQuery(filter_html).find('option[value="eq"]').attr('data-render') == 'primary'
         assert not PyQuery(filter_html).find('option[value="!eq"]').attr('data-render')
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_multiselect_enum_options(self):
         class PeopleType(Enum):
             bob = 'Bob'
@@ -575,7 +608,7 @@ class TestHtmlRenderer(object):
         assert PyQuery(output).find('input[value="bob"]').attr('checked')
         assert not PyQuery(output).find('input[value="charlie"]').attr('checked')
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_confirm_export(self):
         g = PeopleGrid()
         assert json.loads(g.html.confirm_export()) == {'confirm_export': False, 'record_count': 3}
@@ -586,20 +619,20 @@ class TestHtmlRenderer(object):
         g.unconfirmed_export_limit = None
         assert json.loads(g.html.confirm_export()) == {'confirm_export': False, 'record_count': 3}
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_grid_rendering(self):
         g = PeopleGrid()
         # really just making sure no exceptions come through at this point
         assert g.html()
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_no_records(self):
         g = PeopleGrid()
         g.set_records([])
         g.html
         assert '<p class="no-records">No records to display</p>' in g.html()
 
-    @inrequest('/thepage')
+    @_inrequest('/thepage')
     def test_no_pager(self):
         class PgNP(PeopleGrid):
             pager_on = False
@@ -626,7 +659,7 @@ class TestHtmlRenderer(object):
         with pytest.raises(RenderLimitExceeded):
             TestGrid().html()
 
-    @inrequest('/thepage?search=foo')
+    @_inrequest('/thepage?search=foo')
     def test_render_search_filter(self):
         g = PeopleGrid()
         g.enable_search = True
@@ -639,7 +672,7 @@ class TestHtmlRenderer(object):
         filter_html = g.html.filtering_fields()
         assert_tag(filter_html, 'input', id='search_input', name='search', type='text', value='foo')
 
-    @inrequest('/thepage?search=foo&dgreset=1')
+    @_inrequest('/thepage?search=foo&dgreset=1')
     def test_render_search_filter_reset(self):
         g = PeopleGrid()
         g.enable_search = True
@@ -676,7 +709,7 @@ class PGPageTotals(PeopleGrid):
 
 
 class TestPageTotals(object):
-    @inrequest('/')
+    @_inrequest('/')
     def test_people_html(self):
         g = PGPageTotals()
         html = g.html()
@@ -690,7 +723,7 @@ class PGGrandTotals(PeopleGrid):
 
 
 class TestGrandTotals(object):
-    @inrequest('/')
+    @_inrequest('/')
     def test_people_html(self):
         g = PGGrandTotals()
         g.html
@@ -699,7 +732,7 @@ class TestGrandTotals(object):
 
 
 class TestFooterRendersCorrectly(object):
-    @inrequest('/')
+    @_inrequest('/')
     def test_people_html_footer(self):
         g = PeopleGrid()
         g.html
@@ -708,7 +741,7 @@ class TestFooterRendersCorrectly(object):
         # make sure we are rendering the separator
         assert '&nbsp;|' in g.html()
 
-    @inrequest('/')
+    @_inrequest('/')
     def test_people_html_footer_only_csv(self):
         g = PeopleCSVGrid()
         g.html
@@ -721,7 +754,7 @@ class PGAllTotals(PeopleGrid):
 
 
 class TestAllTotals(object):
-    @inrequest('/')
+    @_inrequest('/')
     def test_people_html(self):
         g = PGAllTotals()
         html = g.html()
@@ -739,7 +772,7 @@ class PGTotalsStringExpr(PeopleGrid):
 
 
 class TestStringExprTotals:
-    @inrequest('/')
+    @_inrequest('/')
     def test_people_html(self):
         g = PGTotalsStringExpr()
         html = g.html()
@@ -748,6 +781,217 @@ class TestStringExprTotals:
         assert_tag(html, 'td', text='Page Totals (3 records):', class_='totals-label', colspan='7')
 
 
+class TestJSONRenderer:
+    def get_json(self, grid):
+        return json.loads(JSON(grid).render())
+
+    def test_json_format_records(self):
+        status_options = [{
+            'key': status.id,
+            'value': status.label
+        } for status in reversed(Status.list())]
+        expected = {
+            'errors': [],
+            'settings': {
+                'filters': {},
+                'paging': {'pager_on': True, 'on_page': 1, 'per_page': 50},
+                'search_expr': None,
+                'sort': [],
+                'export_to': None,
+            },
+            'spec': {
+                'columns': {
+                    'account_type': 'Account Type',
+                    'createdts': 'Created',
+                    'due_date': 'Due Date',
+                    'emails': 'Emails',
+                    'firstname': 'First Name',
+                    'full_name': 'Full Name',
+                    'inactive': 'Active',
+                    'numericcol': 'Number',
+                    'status': 'Status',
+                },
+                'column_groups': [
+                    {'columns': ['firstname', 'inactive'], 'label': 'foo'}
+                ],
+                'enable_search': True,
+                'enable_sort': True,
+                'export_targets': ['xls', 'xlsx'],
+                'filters': {
+                    'account_type': {
+                        'operators': [
+                            {'field_type': 'select', 'hint': None, 'key': 'is', 'label': 'is'},
+                            {'field_type': 'select', 'hint': None, 'key': '!is', 'label': 'is not'},
+                            {'field_type': None, 'hint': None, 'key': 'empty', 'label': 'empty'},
+                            {'field_type': None, 'hint': None, 'key': '!empty',
+                             'label': 'not empty'}
+                        ],
+                        'options': [
+                            {'key': 'admin', 'value': 'Admin'},
+                            {'key': 'manager', 'value': 'Manager'},
+                            {'key': 'employee', 'value': 'Employee'}
+                        ],
+                        'primary_op': {'field_type': 'select', 'hint': None, 'key': 'is',
+                                       'label': 'is'}
+                    },
+                    'createdts': {
+                        'operators': [
+                            {'field_type': 'input', 'hint': None, 'key': 'eq', 'label': 'is'},
+                            {'field_type': 'input', 'hint': None, 'key': '!eq', 'label': 'is not'},
+                            {'field_type': 'input', 'hint': None, 'key': 'lte',
+                             'label': 'less than or equal'},
+                            {'field_type': 'input', 'hint': None, 'key': 'gte',
+                             'label': 'greater than or equal'},
+                            {'field_type': '2inputs', 'hint': None, 'key': 'between',
+                             'label': 'between'},
+                            {'field_type': '2inputs', 'hint': None, 'key': '!between',
+                             'label': 'not between'},
+                            {'field_type': 'input', 'hint': 'days', 'key': 'da',
+                             'label': 'days ago'},
+                            {'field_type': 'input', 'hint': 'days', 'key': 'ltda',
+                             'label': 'less than days ago'},
+                            {'field_type': 'input', 'hint': 'days', 'key': 'mtda',
+                             'label': 'more than days ago'},
+                            {'field_type': None, 'hint': None, 'key': 'today', 'label': 'today'},
+                            {'field_type': None, 'hint': None, 'key': 'thisweek',
+                             'label': 'this week'},
+                            {'field_type': 'input', 'hint': 'days', 'key': 'ind',
+                             'label': 'in days'},
+                            {'field_type': 'input', 'hint': 'days', 'key': 'iltd',
+                             'label': 'in less than days'},
+                            {'field_type': 'input', 'hint': 'days', 'key': 'imtd',
+                             'label': 'in more than days'},
+                            {'field_type': None, 'hint': None, 'key': 'empty', 'label': 'empty'},
+                            {'field_type': None, 'hint': None, 'key': '!empty',
+                             'label': 'not empty'},
+                            {'field_type': None, 'hint': None, 'key': 'thismonth',
+                             'label': 'this month'},
+                            {'field_type': None, 'hint': None, 'key': 'lastmonth',
+                             'label': 'last month'},
+                            {'field_type': 'select+input', 'hint': None, 'key': 'selmonth',
+                             'label': 'select month'},
+                            {'field_type': None, 'hint': None, 'key': 'thisyear',
+                             'label': 'this year'}
+                        ],
+                        'primary_op': None
+                    },
+                    'firstname': {
+                        'operators': [
+                            {'field_type': 'input', 'hint': None, 'key': 'eq', 'label': 'is'},
+                            {'field_type': 'input', 'hint': None, 'key': '!eq',
+                             'label': 'is not'},
+                            {'field_type': 'input', 'hint': None, 'key': 'contains',
+                             'label': 'contains'},
+                            {'field_type': 'input', 'hint': None, 'key': '!contains',
+                             'label': "doesn't contain"},
+                            {'field_type': None, 'hint': None, 'key': 'empty', 'label': 'empty'},
+                            {'field_type': None, 'hint': None, 'key': '!empty',
+                             'label': 'not empty'}
+                        ],
+                        'primary_op': {'field_type': 'input', 'hint': None, 'key': 'contains',
+                                       'label': 'contains'}
+                    },
+                    'status': {
+                        'operators': [
+                            {'field_type': None, 'hint': None, 'key': 'o', 'label': 'open'},
+                            {'field_type': 'select', 'hint': None, 'key': 'is', 'label': 'is'},
+                            {'field_type': 'select', 'hint': None, 'key': '!is', 'label': 'is not'},
+                            {'field_type': None, 'hint': None, 'key': 'c', 'label': 'closed'},
+                            {'field_type': None, 'hint': None, 'key': 'empty', 'label': 'empty'},
+                            {'field_type': None, 'hint': None, 'key': '!empty',
+                             'label': 'not empty'}
+                        ],
+                        'options': status_options,
+                        'primary_op': {'field_type': 'select', 'hint': None, 'key': 'is',
+                                       'label': 'is'}
+                    },
+                },
+            },
+            'state': {
+                'page_count': 1,
+                'record_count': 3,
+                'warnings': [],
+            },
+            'records': [
+                {
+                    'account_type': None,
+                    'createdts': '2012-02-22T10:04:16',
+                    'due_date': '2012-02-04',
+                    'emails': 'email004@example.com, email004@gmail.com',
+                    'firstname': 'fn004',
+                    'full_name': 'fn004 ln004',
+                    'inactive': 'Yes',
+                    'numericcol': '2.13',
+                    'status': None,
+                },
+                {
+                    'account_type': 'Employee',
+                    'createdts': None,
+                    'due_date': None,
+                    'emails': 'email002@example.com, email002@gmail.com',
+                    'firstname': 'fn002',
+                    'full_name': 'fn002 ln002',
+                    'inactive': 'Yes',
+                    'numericcol': '2.13',
+                    'status': 'pending',
+                },
+                {
+                    'account_type': 'Admin',
+                    'createdts': '2012-02-22T10:01:16',
+                    'due_date': '2012-02-01',
+                    'emails': 'email001@example.com, email001@gmail.com',
+                    'firstname': 'fn001',
+                    'full_name': 'fn001 ln001',
+                    'inactive': 'Yes',
+                    'numericcol': '2.13',
+                    'status': 'in process',
+                },
+            ],
+        }
+        grid = PeopleGrid()
+        group = ColumnGroup('foo')
+        grid.column('firstname').group = grid.column('inactive').group = group
+        assert self.get_json(grid) == expected
+
+    def test_warnings(self):
+        grid = PeopleGrid()
+        grid.user_warnings = ['foo', 'bar', 'baz']
+        assert self.get_json(grid)['state']['warnings'] == ['foo', 'bar', 'baz']
+
+    def test_json_format_settings(self):
+        grid = PeopleGrid()
+        firstname = grid.column('firstname')
+        firstname.filter.set('eq', 'bar', 'baz')
+        # Ensure that raw filter value is serialized
+        firstname.filter.value1 = 'bong'
+        firstname.filter.value2 = 'bing'
+        grid.set_paging(20, 2)
+        grid.search_value = 'foo'
+        grid.set_sort('firstname', '-status')
+        assert self.get_json(grid)['settings'] == {
+            'filters': {
+                'firstname': {'op': 'eq', 'value1': 'bar', 'value2': 'baz'},
+            },
+            'paging': {'pager_on': True, 'on_page': 2, 'per_page': 20},
+            'search_expr': 'foo',
+            'sort': [
+                {'key': 'firstname', 'flag_desc': False},
+                {'key': 'status', 'flag_desc': True},
+            ],
+            'export_to': None,
+        }
+
+    def test_json_format_arrow(self):
+        ArrowRecord.query.delete()
+        ArrowRecord.testing_create(
+            created_utc=arrow.Arrow(2016, 8, 10, 1, 2, 3)
+        )
+        reloaded = self.get_json(ArrowGrid())
+        assert reloaded['records'] == [
+            {'created_utc': '2016-08-10T01:02:03+00:00'}
+        ]
+
+
 class TestXLSRenderer(object):
 
     def test_some_basics(self):
@@ -1038,7 +1282,7 @@ class TestCSVRenderer(object):
 
 
 class TestHideSection(object):
-    @inrequest('/')
+    @_inrequest('/')
     def test_controlls_hidden(self):
         class NoControlBoxGrid(PG):
             hide_controls_box = True
@@ -1048,9 +1292,12 @@ class TestHideSection(object):
 
 
 class TestArrowDate(object):
-    @inrequest('/')
-    def test_arrow_render_html(self):
+
+    def setup_method(self):
         ArrowRecord.query.delete()
+
+    @_inrequest('/')
+    def test_arrow_render_html(self):
         ArrowRecord.testing_create(created_utc=arrow.Arrow(2016, 8, 10, 1, 2, 3))
         g = ArrowGrid()
         assert '<td>08/10/2016 01:02 AM</td>' in g.html(), g.html()
@@ -1058,10 +1305,9 @@ class TestArrowDate(object):
         g.column('created_utc').html_format = 'YYYY-MM-DD HH:mm:ss ZZ'
         assert '<td>2016-08-10 01:02:03 +00:00</td>' in g.html(), g.html()
 
-    @inrequest('/')
+    @_inrequest('/')
     def test_arrow_timezone(self):
         # regardless of timezone given, ArrowType stored as UTC and will display that way
-        ArrowRecord.query.delete()
         ArrowRecord.testing_create(created_utc=arrow.Arrow(2016, 8, 10, 1, 2, 3).to('US/Pacific'))
         g = ArrowGrid()
         assert '<td>08/10/2016 01:02 AM</td>' in g.html(), g.html()
@@ -1069,8 +1315,18 @@ class TestArrowDate(object):
         g.column('created_utc').html_format = 'YYYY-MM-DD HH:mm:ss ZZ'
         assert '<td>2016-08-10 01:02:03 +00:00</td>' in g.html(), g.html()
 
+    def test_filter_handles_arrow(self):
+        ArrowRecord.testing_create(created_utc=arrow.Arrow(2016, 8, 10, 1, 2, 3))
+        g = ArrowGrid()
+        g.column('created_utc').filter = DateFilter(g.column('created_utc').expr)
+        g.apply_qs_args(grid_args={
+            'op(created_utc)': 'eq',
+            'v1(created_utc)': '2018-01-01'
+        })
+        assert g.column('created_utc').filter.is_active
+        g.record_count
+
     def test_xls(self):
-        ArrowRecord.query.delete()
         ArrowRecord.testing_create(created_utc=arrow.Arrow(2016, 8, 10, 1, 2, 3))
         g = ArrowGrid()
         buffer = io.BytesIO()
