@@ -7,17 +7,23 @@ from os import path
 import arrow
 import flask
 import pytest
-from mock import mock
+from mock import mock, MagicMock
 import sqlalchemy.sql as sasql
 from werkzeug.datastructures import MultiDict
 
 from webgrid import Column, BoolColumn, YesNoColumn
-from webgrid.extensions import RequestArgsLoader, WebSessionArgsLoader
+from webgrid.extensions import (
+    RequestArgsLoader,
+    RequestFormLoader,
+    RequestJsonLoader,
+    WebSessionArgsLoader,
+    lazy_gettext as _
+)
 from webgrid.filters import FilterBase, TextFilter, IntFilter, AggregateIntFilter
-from webgrid.testing import assert_in_query, assert_not_in_query, query_to_str
+from webgrid.testing import assert_in_query, assert_not_in_query
 from webgrid_ta.model.entities import Person, Status, Stopwatch, db
 from webgrid_ta.grids import Grid, PeopleGrid, PeopleGridByConfig
-from .helpers import inrequest
+from .helpers import _inrequest
 from webgrid.renderers import CSV
 
 
@@ -29,6 +35,9 @@ class TestGrid(object):
         # demonstrate grid operations when multiple column expressions have the same key
         Column('Person ID', Person.id, IntFilter)
         Column('Stopwatch ID', Stopwatch.id, IntFilter)
+        Column('No Expression')
+        Column(_('No Expression'))
+        Column('')
 
         query_joins = [(Stopwatch, Stopwatch.id > 0)]
 
@@ -175,6 +184,33 @@ class TestGrid(object):
         g = CTG()
         assert_in_query(g, 'ORDER BY persons.last_name')
 
+    def test_column_reorder(self):
+        class CTG(Grid):
+            Column('First Name', Person.firstname, TextFilter)
+            Column('Last Name', Person.lastname, TextFilter)
+
+        grid = CTG()
+        grid.set_column_order(('lastname', 'firstname'))
+        assert grid.columns[-1].label == 'First Name'
+        assert grid.columns[0].label == 'Last Name'
+
+        with pytest.raises(Exception, match="Keys not recognized on grid: {'foo'}"):
+            grid.set_column_order(('foo', ))
+
+    def test_add_column(self):
+        class CTG(Grid):
+            Column('First Name', Person.firstname, TextFilter)
+
+        grid = CTG()
+        grid.add_column(
+            Column('Last Name', Person.lastname, TextFilter)
+        )
+
+        assert len(grid.columns) == 2
+        colinst = grid.column('lastname')
+        assert colinst.grid is grid
+        assert colinst.key == 'lastname'
+
     def test_filter_class(self):
         class CTG(Grid):
             Column('First Name', Person.firstname, TextFilter)
@@ -272,21 +308,22 @@ class TestGrid(object):
     def test_paging(self):
         g = self.TG()
         if db.engine.dialect.name == 'mssql':
-            assert_in_query(g, 'SELECT TOP 50 ')
+            assert_in_query(g, 'OFFSET 0 ROWS')
+            assert_in_query(g, 'FETCH FIRST 50 ROWS ONLY')
         else:
             assert_in_query(g, 'LIMIT 50 OFFSET 0')
 
         g.set_paging(2, 1)
         if db.engine.dialect.name == 'mssql':
-            assert_in_query(g, 'SELECT TOP 2 ')
+            assert_in_query(g, 'OFFSET 0 ROWS')
+            assert_in_query(g, 'FETCH FIRST 2 ROWS ONLY')
         else:
             assert_in_query(g, 'LIMIT 2 OFFSET 0')
 
         g.set_paging(10, 5)
         if db.engine.dialect.name == 'mssql':
-            assert_in_query(g, 'WHERE mssql_rn > 40 AND mssql_rn <= 10 + 40')
-            assert_in_query(g, 'SELECT persons.firstname AS firstname, ROW_NUMBER() '
-                               'OVER (ORDER BY persons.firstname) AS mssql_rn')
+            assert_in_query(g, 'OFFSET 40 ROWS')
+            assert_in_query(g, 'FETCH FIRST 10 ROWS ONLY')
         else:
             assert_in_query(g, 'LIMIT 10 OFFSET 40')
 
@@ -450,6 +487,15 @@ class TestGrid(object):
         with pytest.raises(Exception, match='bad filter search expression: foo is not callable'):
             CTG().search_expression_generators
 
+    def test_search_blank(self):
+        class CTG(Grid):
+            Column('First Name', Person.firstname, TextFilter)
+            Column('Last Name', Person.lastname, TextFilter)
+
+        g = CTG()
+        g.search_value = ''
+        assert_not_in_query(g.build_query(), 'WHERE')
+
     def test_search_query(self):
         class CTG(Grid):
             Column('First Name', Person.firstname, TextFilter)
@@ -521,6 +567,13 @@ class TestGrid(object):
         assert grid.column('id').expr == Person.id
         assert grid.column('id_1').expr == Stopwatch.id
 
+    def test_no_expression_column_key(self):
+        """Test that columns without an expression get a key"""
+        grid = self.KeyGrid()
+        assert grid.column('no_expression')
+        assert grid.column('no_expression_1')
+        assert grid.column('unnamed_expression')
+
     def test_column_keys_unique_query_default_sort(self):
         grid = self.KeyGrid()
         grid.query_default_sort = 'id'
@@ -530,19 +583,19 @@ class TestGrid(object):
         grid.query_default_sort = Person.id
         assert_in_query(grid, 'ORDER BY persons.id')
 
-    @inrequest('/foo?op(id)=gte&v1(id)=0')
+    @_inrequest('/foo?op(id)=gte&v1(id)=0')
     def test_column_keys_unique_filter_persons(self):
         grid = self.KeyGrid()
         grid.apply_qs_args()
         assert_in_query(grid, 'WHERE persons.id >= 0')
 
-    @inrequest('/foo?op(id_1)=gte&v1(id_1)=0')
+    @_inrequest('/foo?op(id_1)=gte&v1(id_1)=0')
     def test_column_keys_unique_filter_stopwatches(self):
         grid = self.KeyGrid()
         grid.apply_qs_args()
         assert_in_query(grid, 'WHERE stopwatches.id >= 0')
 
-    @inrequest('/foo?sort1=id_1&sort2=id')
+    @_inrequest('/foo?sort1=id_1&sort2=id')
     def test_column_keys_unique_sort(self):
         grid = self.KeyGrid()
         grid.apply_qs_args()
@@ -561,7 +614,7 @@ class TestQueryStringArgs(object):
         Person.testing_create('bob')
         Person.testing_create()
 
-    @inrequest('/foo?dg_perpage=1&dg_onpage=2')
+    @_inrequest('/foo?dg_perpage=1&dg_onpage=2')
     @pytest.mark.parametrize('grid_cls', [PeopleGrid, PeopleGridByConfig])
     def test_qs_prefix(self, grid_cls):
         pg = grid_cls(qs_prefix='dg_')
@@ -569,7 +622,7 @@ class TestQueryStringArgs(object):
         assert pg.on_page == 2
         assert pg.per_page == 1
 
-    @inrequest('/foo?perpage=1&onpage=2')
+    @_inrequest('/foo?perpage=1&onpage=2')
     def test_qs_paging(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -578,11 +631,12 @@ class TestQueryStringArgs(object):
 
         # make sure the corret values get applied to the query
         if db.engine.dialect.name == 'mssql':
-            assert 'WHERE mssql_rn > 1 AND mssql_rn <= 1 + 1' in query_to_str(pg.build_query())
+            assert_in_query(pg, 'OFFSET 1 ROWS')
+            assert_in_query(pg, 'FETCH FIRST 1 ROWS ONLY')
         else:
-            assert 'LIMIT 1 OFFSET 1' in query_to_str(pg.build_query())
+            assert_in_query(pg, 'LIMIT 1 OFFSET 1')
 
-    @inrequest('/foo?perpage=5&onpage=foo')
+    @_inrequest('/foo?perpage=5&onpage=foo')
     def test_qs_onpage_invalid(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -590,7 +644,7 @@ class TestQueryStringArgs(object):
         assert pg.per_page == 5
         assert pg.user_warnings[0] == '"onpage" grid argument invalid, ignoring'
 
-    @inrequest('/foo?perpage=5&onpage=-1')
+    @_inrequest('/foo?perpage=5&onpage=-1')
     def test_qs_onpage_negative(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -598,7 +652,7 @@ class TestQueryStringArgs(object):
         assert pg.per_page == 5
         assert len(pg.user_warnings) == 0
 
-    @inrequest('/foo?perpage=1&onpage=100')
+    @_inrequest('/foo?perpage=1&onpage=100')
     def test_qs_onpage_too_high(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -609,7 +663,7 @@ class TestQueryStringArgs(object):
         # the records should be the same as if we were on the last page
         assert len(pg.records) == 1
 
-    @inrequest('/foo?perpage=foo&onpage=2')
+    @_inrequest('/foo?perpage=foo&onpage=2')
     def test_qs_perpage_invalid(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -617,7 +671,7 @@ class TestQueryStringArgs(object):
         assert pg.per_page == 1
         assert pg.user_warnings[0] == '"perpage" grid argument invalid, ignoring'
 
-    @inrequest('/foo?perpage=-1&onpage=2')
+    @_inrequest('/foo?perpage=-1&onpage=2')
     def test_qs_perpage_negative(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -625,21 +679,21 @@ class TestQueryStringArgs(object):
         assert pg.per_page == 1
         assert len(pg.user_warnings) == 0
 
-    @inrequest('/foo?sort1=foo&sort2=firstname&sort3=-status')
+    @_inrequest('/foo?sort1=foo&sort2=firstname&sort3=-status')
     def test_qs_sorting(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
         assert pg.order_by == [('firstname', False), ('status', True)]
         assert pg.user_warnings[0] == 'can\'t sort on invalid key "foo"'
 
-    @inrequest('/foo?sort1=fullname&sort2=firstname&sort3=-status')
+    @_inrequest('/foo?sort1=fullname&sort2=firstname&sort3=-status')
     def test_qs_sorting_not_allowed(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
         assert pg.order_by == [('firstname', False), ('status', True)]
         assert pg.user_warnings[0] == 'can\'t sort on invalid key "fullname"'
 
-    @inrequest('/foo?sort1=')
+    @_inrequest('/foo?sort1=')
     def test_qs_sorting_ignores_emptystring(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -649,11 +703,11 @@ class TestQueryStringArgs(object):
     def test_qs_filtering(self):
         first_id = Status.query.filter_by(label='pending').one().id
         second_id = Status.query.filter_by(label='in process').one().id
-        with flask.current_app.test_request_context(
+        pg = PeopleGrid()
+        with pg.manager.test_request_context(
             '/foo?op(firstname)=eq&v1(firstname)=fn001&op(status)=is'
             f'&v1(status)={first_id}&v1(status)={second_id}'
         ):
-            pg = PeopleGrid()
             pg.apply_qs_args()
         assert pg.columns[0].filter.op == 'eq'
         assert pg.columns[0].filter.value1 == 'fn001'
@@ -663,7 +717,7 @@ class TestQueryStringArgs(object):
         assert pg.columns[4].filter.value1 == [first_id, second_id]
         assert pg.columns[4].filter.value2 is None
 
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_qs_filtering_default_op(self):
         from webgrid_ta.grids import DefaultOpGrid
         pg = DefaultOpGrid()
@@ -671,7 +725,7 @@ class TestQueryStringArgs(object):
         assert pg.columns[0].filter.op == 'eq'
         assert pg.columns[0].filter.value1 is None
 
-    @inrequest('/foo?op(firstname)=!eq&v1(firstname)=bob')
+    @_inrequest('/foo?op(firstname)=!eq&v1(firstname)=bob')
     def test_qs_filtering_default_op_override(self):
         from webgrid_ta.grids import DefaultOpGrid
         pg = DefaultOpGrid()
@@ -679,7 +733,7 @@ class TestQueryStringArgs(object):
         assert pg.columns[0].filter.op == '!eq'
         assert pg.columns[0].filter.value1 == 'bob'
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_qs_paging_doesnt_get_page_count_before_filters_are_handled(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -689,13 +743,13 @@ class TestQueryStringArgs(object):
         # page count gets cached.
         assert pg.record_count == 2
 
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_qs_no_session(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
         assert pg.column('firstname').filter.op is None
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_qs_keyed_session(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -708,7 +762,7 @@ class TestQueryStringArgs(object):
         pg.apply_qs_args()
         assert pg.column('firstname').filter.op == 'eq'
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_grid_args_ignores_url(self):
         pg = PeopleGrid()
         pg.apply_qs_args(grid_args=MultiDict({
@@ -718,7 +772,7 @@ class TestQueryStringArgs(object):
         assert pg.column('firstname').filter.op == 'contains'
         assert pg.column('firstname').filter.value1 == 'bill'
 
-    @inrequest('/foo?op(firstname)=&v1(firstname)=foo&op(status)=&v1(status)=1')
+    @_inrequest('/foo?op(firstname)=&v1(firstname)=foo&op(status)=&v1(status)=1')
     def test_qs_blank_operator(self):
         pg = PeopleGrid()
         pg.apply_qs_args()
@@ -730,7 +784,7 @@ class TestQueryStringArgs(object):
         assert pg.columns[4].filter.value1 is None
         assert pg.columns[4].filter.value2 is None
 
-    @inrequest('/foo?sort1=legacycol1&sort2=legacycol2')
+    @_inrequest('/foo?sort1=legacycol1&sort2=legacycol2')
     def test_sa_expr_sort(self):
         class AGrid(Grid):
             Column('First Name', 'firstname')
@@ -750,25 +804,25 @@ class TestQueryStringArgs(object):
         # this will fail if we are not using SA expressions correctly to sort
         g.records
 
-    @inrequest('/thepage?export_to=xls')
+    @_inrequest('/thepage?export_to=xls')
     def test_export_to_xls(self):
         g = PeopleGrid()
         g.apply_qs_args()
         assert g.export_to == 'xls'
 
-    @inrequest('/thepage?export_to=xlsx')
+    @_inrequest('/thepage?export_to=xlsx')
     def test_export_to_xlsx(self):
         g = PeopleGrid()
         g.apply_qs_args()
         assert g.export_to == 'xlsx'
 
-    @inrequest('/thepage?export_to=foo')
+    @_inrequest('/thepage?export_to=foo')
     def test_export_to_unrecognized(self):
         g = PeopleGrid()
         g.apply_qs_args()
         assert g.export_to is None
 
-    @inrequest('/foo?op(id)=eq&v1(id)=d')
+    @_inrequest('/foo?op(id)=eq&v1(id)=d')
     def test_exc_msg_in_warnings(self):
         class TGrid(Grid):
             Column('T', Person.id, IntFilter)
@@ -776,21 +830,21 @@ class TestQueryStringArgs(object):
         g.apply_qs_args()
         assert g.user_warnings[0] == 'T: Please enter an integer value'
 
-    @inrequest('/foo?search=bar')
+    @_inrequest('/foo?search=bar')
     def test_qs_search(self):
         g = PeopleGrid()
         g.enable_search = True
         g.apply_qs_args()
         assert g.search_value == 'bar'
 
-    @inrequest('/foo?search=bar')
+    @_inrequest('/foo?search=bar')
     def test_qs_search_disabled(self):
         g = PeopleGrid()
         g.enable_search = False
         g.apply_qs_args()
         assert g.search_value is None
 
-    @inrequest('/foo?search=bar')
+    @_inrequest('/foo?search=bar')
     def test_qs_no_searchable_columns(self):
         class TG(Grid):
             Column('First Name', Person.firstname)
@@ -801,43 +855,135 @@ class TestQueryStringArgs(object):
         assert g.search_value is None
 
 
-class TestRequestArgsLoader:
+class GridPrefixTestBase:
     def test_passthru(self):
         source_args = MultiDict([('foo', 'bar'), ('baz', 'bin')])
         source_args_copy = source_args.copy()
         grid = PeopleGrid()
-        loader = RequestArgsLoader(grid.manager)
-        result = loader.get_args(grid, source_args)
+        setattr(grid.manager, self.manager_arg_method, MagicMock(return_value=source_args))
+        loader = self.loader_cls(grid.manager)
+        result = loader.get_args(grid, MultiDict())
         assert result == source_args == source_args_copy
 
+    def test_merge_with_previous(self):
+        source_args = MultiDict([('foo', 'bar'), ('baz', 'bin')])
+        previous_args = MultiDict([('foo', 'fog'), ('bong', 'bing')])
+        grid = PeopleGrid()
+        setattr(grid.manager, self.manager_arg_method, MagicMock(return_value=source_args))
+        loader = self.loader_cls(grid.manager)
+        result = loader.get_args(grid, previous_args)
+        assert result == MultiDict(
+            [('foo', 'bar'), ('foo', 'fog'), ('baz', 'bin'), ('bong', 'bing')]
+        )
+
     def test_qs_prefix_filter(self):
         source_args = MultiDict([('foo', 'bar'), ('baz', 'bin'), ('boo', 'hoo'),
                                  ('baz', 'bid')])
         source_args_copy = source_args.copy()
         grid = PeopleGrid(qs_prefix='b')
-        loader = RequestArgsLoader(grid.manager)
-        result = loader.get_args(grid, source_args)
+        setattr(grid.manager, self.manager_arg_method, MagicMock(return_value=source_args))
+        loader = self.loader_cls(grid.manager)
+        result = loader.get_args(grid, MultiDict())
         assert source_args == source_args_copy
         assert result == MultiDict([('az', 'bin'), ('az', 'bid'), ('oo', 'hoo')])
 
     def test_reset(self):
         source_args = MultiDict([('foo', 'bar'), ('baz', 'bin'), ('dgreset', '1')])
         grid = PeopleGrid()
-        loader = RequestArgsLoader(grid.manager)
-        result = loader.get_args(grid, source_args)
+        setattr(grid.manager, self.manager_arg_method, MagicMock(return_value=source_args))
+        loader = self.loader_cls(grid.manager)
+        result = loader.get_args(grid, MultiDict())
         assert result == MultiDict([('dgreset', 1)])
 
     def test_reset_with_session_key(self):
         source_args = MultiDict([('foo', 'bar'), ('baz', 'bin'), ('dgreset', '1'),
                                  ('session_key', '123')])
         grid = PeopleGrid()
-        loader = RequestArgsLoader(grid.manager)
-        result = loader.get_args(grid, source_args)
+        setattr(grid.manager, self.manager_arg_method, MagicMock(return_value=source_args))
+        loader = self.loader_cls(grid.manager)
+        result = loader.get_args(grid, MultiDict())
         assert result == MultiDict([('dgreset', 1), ('session_key', '123')])
 
 
+class TestRequestFormLoader(GridPrefixTestBase):
+    loader_cls = RequestFormLoader
+    manager_arg_method = 'request_form_args'
+
+
+class TestRequestArgsLoader(GridPrefixTestBase):
+    loader_cls = RequestArgsLoader
+    manager_arg_method = 'request_url_args'
+
+
+class TestRequestJsonLoader:
+    def ok_values(self):
+        return {
+            'search_expr': 'foo',
+            'filters': {
+                'test': {'op': 'eq', 'value1': 'toast', 'value2': 'taft'},
+                'test2': {'op': 'in', 'value1': 'tarp', 'value2': None},
+            },
+            'paging': {'on_page': 2, 'per_page': 20},
+            'sort': [{'key': 'bar', 'flag_desc': False}, {'key': 'baz', 'flag_desc': True}],
+        }
+
+    def test_load(self):
+        data = self.ok_values()
+        grid = PeopleGrid()
+        grid.manager.request_json = MagicMock(return_value=data)
+        loader = RequestJsonLoader(grid.manager)
+        result = loader.get_args(grid, MultiDict())
+        assert result == MultiDict([
+            ('search', 'foo'),
+            ('op(test)', 'eq'),
+            ('v1(test)', 'toast'),
+            ('v2(test)', 'taft'),
+            ('op(test2)', 'in'),
+            ('v1(test2)', 'tarp'),
+            ('onpage', 2),
+            ('perpage', 20),
+            ('sort1', 'bar'),
+            ('sort2', '-baz'),
+            ('export_to', None),
+        ])
+
+    def test_merge_with_previous(self):
+        data = self.ok_values()
+        grid = PeopleGrid()
+        grid.manager.request_json = MagicMock(return_value=data)
+        loader = RequestJsonLoader(grid.manager)
+        result = loader.get_args(grid, MultiDict([
+            ('search', 'oof'),
+            ('onpage', 1),
+            ('sort3', 'bong'),
+        ]))
+        assert result == MultiDict([
+            ('search', 'foo'),
+            ('search', 'oof'),
+            ('op(test)', 'eq'),
+            ('v1(test)', 'toast'),
+            ('v2(test)', 'taft'),
+            ('op(test2)', 'in'),
+            ('v1(test2)', 'tarp'),
+            ('onpage', 2),
+            ('onpage', 1),
+            ('perpage', 20),
+            ('sort1', 'bar'),
+            ('sort2', '-baz'),
+            ('sort3', 'bong'),
+            ('export_to', None),
+        ])
+
+    def test_load_empty(self):
+        grid = PeopleGrid()
+        grid.manager.request_json = MagicMock(return_value=None)
+        loader = RequestJsonLoader(grid.manager)
+        result = loader.get_args(grid, MultiDict())
+        assert result == MultiDict()
+
+
 class TestWebSessionArgsLoader:
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_no_session_present(self):
         source_args = MultiDict([('foo', 'bar'), ('baz', 'bin')])
         source_args_copy = source_args.copy()
@@ -851,7 +997,7 @@ class TestWebSessionArgsLoader:
             ('__foreign_session_loaded__', False),
         ])
 
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_empty_session_not_stored(self):
         source_args = MultiDict([])
         grid = PeopleGrid()
@@ -860,7 +1006,7 @@ class TestWebSessionArgsLoader:
         assert '_PeopleGrid' not in flask.session['dgsessions']
         assert grid.session_key not in flask.session['dgsessions']
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob')
     def test_matching_session_not_stored(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -871,14 +1017,14 @@ class TestWebSessionArgsLoader:
         assert pg.session_key in flask.session['dgsessions']
         assert pg2.session_key not in flask.session['dgsessions']
 
-    @inrequest('/foo?dgreset=1&op(firstname)=eq&v1(firstname)=bob')
+    @_inrequest('/foo?dgreset=1&op(firstname)=eq&v1(firstname)=bob')
     def test_reset_result(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
         args = loader.get_args(pg, flask.request.args)
         assert args == MultiDict([('dgreset', 1), ('session_key', None)])
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_reset_removes_session(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -892,7 +1038,7 @@ class TestWebSessionArgsLoader:
         assert '_PeopleGrid' not in flask.session['dgsessions']
         assert pg.session_key not in flask.session['dgsessions']
 
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_session_load_from_none(self):
         # test backwards compatibility for multidict load
         flask.session['dgsessions'] = {}
@@ -901,7 +1047,7 @@ class TestWebSessionArgsLoader:
         args = loader.get_session_store(grid, MultiDict())
         assert args == MultiDict([])
 
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_session_load_from_multidict(self):
         # test backwards compatibility for multidict load
         flask.session['dgsessions'] = {
@@ -912,7 +1058,7 @@ class TestWebSessionArgsLoader:
         args = loader.get_session_store(grid, MultiDict())
         assert args == MultiDict([('a', 'b'), ('a', 'c')])
 
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_session_load_from_dict(self):
         # test backwards compatibility for dict load
         flask.session['dgsessions'] = {
@@ -923,7 +1069,7 @@ class TestWebSessionArgsLoader:
         args = loader.get_session_store(grid, MultiDict())
         assert args == MultiDict([('a', 'b'), ('c', 'd')])
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_default_session(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -933,7 +1079,7 @@ class TestWebSessionArgsLoader:
         args = loader.get_args(pg2, MultiDict())
         assert args['op(firstname)'] == 'eq'
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_keyed_session(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -947,7 +1093,7 @@ class TestWebSessionArgsLoader:
         args = loader.get_args(pg, flask.request.args)
         assert args['op(firstname)'] == 'eq'
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100&sort1=foo')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100&sort1=foo')
     def test_page_args_always_applied(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -965,7 +1111,7 @@ class TestWebSessionArgsLoader:
         assert args['sort1'] == 'foo'
         assert '["perpage", "15"]' in flask.session['dgsessions'][pg.session_key]
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100&sort1=foo')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100&sort1=foo')
     def test_sort_args_always_applied(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -981,7 +1127,7 @@ class TestWebSessionArgsLoader:
         assert args['sort1'] == 'bar'
         assert '["sort1", "bar"]' in flask.session['dgsessions'][pg.session_key]
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_keyed_session_with_export(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -1000,7 +1146,7 @@ class TestWebSessionArgsLoader:
         assert args['export_to'] == 'xls'
         assert 'export_to' not in flask.session['dgsessions'][pg.session_key]
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_keyed_session_with_override(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -1018,7 +1164,7 @@ class TestWebSessionArgsLoader:
         assert args['op(createdts)'] == '!eq'
         assert args['v1(createdts)'] == '2017-05-06'
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_keyed_session_without_override(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
@@ -1035,13 +1181,14 @@ class TestWebSessionArgsLoader:
         assert args['op(createdts)'] == '!eq'
         assert args['v1(createdts)'] == '2017-05-06'
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_apply_prevents_session_load(self):
         pg = PeopleGrid()
         loader = WebSessionArgsLoader(pg.manager)
         loader.get_args(pg, flask.request.args)
         flask.request.args = MultiDict([
             ('session_key', pg.session_key),
+            ('op(createdts)', 'today'),
             ('apply', None),
         ])
         pg2 = PeopleGrid()
@@ -1050,7 +1197,7 @@ class TestWebSessionArgsLoader:
         assert 'v1(firstname)' not in args
         assert 'apply' not in flask.session['dgsessions'][pg.session_key]
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_expired_session_cleaned_up(self):
         with mock.patch(
             'webgrid.extensions.arrow.utcnow',
@@ -1074,7 +1221,7 @@ class TestWebSessionArgsLoader:
         assert '_PeopleGrid' not in flask.session['dgsessions']
         assert pg.session_key not in flask.session['dgsessions']
 
-    @inrequest('/foo')
+    @_inrequest('/foo')
     def test_expired_session_no_stamp(self):
         flask.session['dgsessions'] = {
             '_PeopleGrid': MultiDict([('a', 'b'), ('a', 'c')])
@@ -1085,7 +1232,7 @@ class TestWebSessionArgsLoader:
         loader.cleanup_expired_sessions()
         assert flask.session['dgsessions']['_PeopleGrid'] == MultiDict([('a', 'b'), ('a', 'c')])
 
-    @inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
+    @_inrequest('/foo?op(firstname)=eq&v1(firstname)=bob&perpage=1&onpage=100')
     def test_expired_session_no_max_hours(self):
         with mock.patch(
             'webgrid.extensions.arrow.utcnow',
